import { Component, ViewChild, ElementRef } from "@angular/core";

import { FixedResults } from "../../results/fixed-results";
import { NgParameter } from "../../formulaire/ngparam";
import { CalculatorResults } from "../../results/calculator-results";
import { I18nService } from "../../services/internationalisation/internationalisation.service";
import { ApplicationSetupService } from "../../services/app-setup/app-setup.service";
import * as XLSX from "xlsx";
import { Structure } from "jalhyd";
import { FormulaireService } from "../../services/formulaire/formulaire.service";

@Component({
    selector: "fixed-results",
    templateUrl: "./fixed-results.component.html",
    styleUrls: [
        "./fixed-results.component.scss"
    ]
})
export class FixedResultsComponent {
    /** résultats non mis en forme */
    private _fixedResults: FixedResults;

    @ViewChild("tableContainer", { static: false })
    table: ElementRef;

    public tableColumns = [ "parametre", "valeur" ];

    constructor(
        private intlService: I18nService,
        private appSetupService: ApplicationSetupService,
        private formService: FormulaireService
    ) { }

    public set results(r: FixedResults) {
        this._fixedResults = r;
    }

    private get hasParameterResult(): boolean {
        return this._fixedResults && this._fixedResults.hasResults;
    }

    private get uitextParamFixes() {
        return this.intlService.localizeText("INFO_CALCULATOR_PARAMFIXES");
    }

    private get uitextValeurs() {
        return this.intlService.localizeText("INFO_CALCULATOR_VALEURS");
    }

    private get uitextExportAsSpreadsheet() {
        return this.intlService.localizeText("INFO_RESULTS_EXPORT_AS_SPREADSHEET");
    }

    private get fixedParams() {
        return this._fixedResults && this._fixedResults.fixedParameters;
    }

    public get hasFixedParameters() {
        return this._fixedResults && this._fixedResults.fixedParameters.length > 0;
    }

    private formattedLabel(p: NgParameter): string {
        return CalculatorResults.paramLabel(p, false);
    }

    private formattedValue(p: NgParameter): string {
        const nDigits = this.appSetupService.displayDigits;
        return p.getValue().toFixed(nDigits);
    }

    /**
     * Returns a set of parameters and results for mat-table
     */
    public get dataSet() {
        const data = [];
        // 1. fixed parameters
        for (const fp of this.fixedParams) {
            let label = this.formattedLabel(fp);
            // add structure position before label
            if (fp.paramDefinition.parentNub instanceof Structure) {
                const pos = fp.paramDefinition.parentNub.findPositionInParent();
                label = this.intlService.localizeText("INFO_OUVRAGE") + " n°" + (pos + 1) + ": " + label;
            }
            data.push({
                label: label,
                value: this.formattedValue(fp),
                isCalcResult: false // for CSS
            });
        }
        // 2. calculation results
        const res = this._fixedResults.result;
        if (
            res
            && res.resultElements.length > 0
            && res.resultElement
            && res.resultElement.count() > 0
        ) {
            const sn = this._fixedResults.result.sourceNub;

            // 2.1 all results
            for (const k of res.resultElement.keys) {
                const er: number = res.resultElement.getValue(k);
                // calculator type for translation
                let ct = sn.calcType;
                if (sn.parent) {
                    ct = sn.parent.calcType;
                }
                data.push({
                    label: this.formService.expandVariableNameAndUnit(ct, k),
                    value: this.intlService.formatResult(k, er),
                    isCalcResult: true // for CSS
                });
            }

            // 2.2. children results
            for (const c of sn.getChildren()) {
                if (c.result) {
                    for (const k of c.result.resultElement.keys) {
                        const er: number = c.result.resultElement.getValue(k);
                        // calculator type for translation
                        let ct = sn.calcType;
                        if (sn.parent) {
                            ct = sn.parent.calcType;
                        }
                        data.push({
                            label: this.intlService.localizeText("INFO_OUVRAGE_N")
                                    + (c.findPositionInParent() + 1) + " : "
                                    + this.formService.expandVariableNameAndUnit(ct, k),
                            value: this.intlService.formatResult(k, er),
                            isCalcResult: true // for CSS
                        });
                    }
                }
            }
        }
        return data;
    }

    public exportAsSpreadsheet() {
        // automagic <table> extraction
        const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(this.table.nativeElement);
        const wb: XLSX.WorkBook = XLSX.utils.book_new();
        XLSX.utils.book_append_sheet(wb, ws, "default");
        // save and download
        XLSX.writeFile(wb, "FixedResults.xlsx");
    }
}
